@@ -15,13 +15,19 @@ del get_versions
 
 # feature_data needs to be imported before feature_table to avoid circular
 # import.
-importlib.import_module('q2_types.multiplexed_sequences')
 importlib.import_module('q2_types.feature_data')
+importlib.import_module('q2_types.per_sample_sequences')
+importlib.import_module('q2_types.feature_map')
 importlib.import_module('q2_types.feature_table')
 importlib.import_module('q2_types.distance_matrix')
 importlib.import_module('q2_types.tree')
 importlib.import_module('q2_types.ordination')
 importlib.import_module('q2_types.sample_data')
-importlib.import_module('q2_types.per_sample_sequences')
 importlib.import_module('q2_types.bowtie2')
 importlib.import_module('q2_types.metadata')
+importlib.import_module('q2_types.multiplexed_sequences')
+importlib.import_module('q2_types.kraken2')
+importlib.import_module('q2_types.feature_data_mag')
+importlib.import_module('q2_types.genome_data')
+importlib.import_module('q2_types.kaiju')
+importlib.import_module('q2_types.reference_db')
