@@ -16,7 +16,10 @@ from ._format import (
     AlignedDNASequencesDirectoryFormat, DifferentialFormat,
     DifferentialDirectoryFormat, FASTAFormat, AlignedFASTAFormatMixin,
     AlignedProteinSequencesDirectoryFormat, ProteinSequencesDirectoryFormat,
-    ProteinFASTAFormat, AlignedProteinFASTAFormat, RNASequencesDirectoryFormat,
+    MixedCaseProteinSequencesDirectoryFormat,
+    MixedCaseAlignedProteinSequencesDirectoryFormat,
+    ProteinFASTAFormat, AlignedProteinFASTAFormat, MixedCaseProteinFASTAFormat,
+    MixedCaseAlignedProteinFASTAFormat, RNASequencesDirectoryFormat,
     RNAFASTAFormat, AlignedRNAFASTAFormat, AlignedRNASequencesDirectoryFormat,
     PairedRNASequencesDirectoryFormat, BLAST6Format, BLAST6DirectoryFormat,
     MixedCaseDNAFASTAFormat, MixedCaseDNASequencesDirectoryFormat,
@@ -24,17 +27,20 @@ from ._format import (
     MixedCaseAlignedDNAFASTAFormat,
     MixedCaseAlignedDNASequencesDirectoryFormat,
     MixedCaseAlignedRNAFASTAFormat,
-    MixedCaseAlignedRNASequencesDirectoryFormat)
+    MixedCaseAlignedRNASequencesDirectoryFormat,
+    SequenceCharacteristicsDirectoryFormat,
+    SequenceCharacteristicsFormat)
 from ._type import (
     FeatureData, Taxonomy, Sequence, PairedEndSequence, AlignedSequence,
     Differential, ProteinSequence, AlignedProteinSequence, RNASequence,
-    AlignedRNASequence, PairedEndRNASequence, BLAST6)
+    AlignedRNASequence, PairedEndRNASequence, BLAST6, SequenceCharacteristics)
 
 # TODO remove these imports when tests are rewritten. Remove from __all__ too
 from ._transformer import (
     NucleicAcidIterator, DNAIterator, PairedDNAIterator, AlignedDNAIterator,
     ProteinIterator, AlignedProteinIterator, RNAIterator, AlignedRNAIterator,
-    PairedRNAIterator)
+    PairedRNAIterator, BarcodePairedSequenceFastqIterator,
+    BarcodeSequenceFastqIterator)
 
 __all__ = [
     'TaxonomyFormat', 'TaxonomyDirectoryFormat', 'HeaderlessTSVTaxonomyFormat',
@@ -59,6 +65,15 @@ __all__ = [
     'MixedCaseAlignedDNAFASTAFormat',
     'MixedCaseAlignedDNASequencesDirectoryFormat',
     'MixedCaseAlignedRNAFASTAFormat',
-    'MixedCaseAlignedRNASequencesDirectoryFormat']
+    'MixedCaseAlignedRNASequencesDirectoryFormat',
+    'BarcodePairedSequenceFastqIterator',
+    'BarcodeSequenceFastqIterator',
+    'MixedCaseProteinFASTAFormat',
+    'MixedCaseAlignedProteinFASTAFormat',
+    'MixedCaseProteinSequencesDirectoryFormat',
+    'MixedCaseAlignedProteinSequencesDirectoryFormat',
+    'SequenceCharacteristics', 'SequenceCharacteristicsDirectoryFormat',
+    'SequenceCharacteristicsFormat'
+    ]
 
 importlib.import_module('q2_types.feature_data._transformer')
