@@ -18,6 +18,10 @@ from q2_types.feature_data import (
     PairedDNASequencesDirectoryFormat, AlignedDNAFASTAFormat,
     AlignedDNASequencesDirectoryFormat, DifferentialDirectoryFormat,
     ProteinFASTAFormat, AlignedProteinFASTAFormat, FASTAFormat,
+    MixedCaseProteinFASTAFormat,
+    MixedCaseAlignedProteinFASTAFormat,
+    MixedCaseAlignedProteinSequencesDirectoryFormat,
+    MixedCaseProteinSequencesDirectoryFormat,
     AlignedProteinSequencesDirectoryFormat, ProteinSequencesDirectoryFormat,
     RNAFASTAFormat, RNASequencesDirectoryFormat, AlignedRNAFASTAFormat,
     AlignedRNASequencesDirectoryFormat, BLAST6DirectoryFormat,
@@ -25,7 +29,9 @@ from q2_types.feature_data import (
     MixedCaseRNAFASTAFormat, MixedCaseRNASequencesDirectoryFormat,
     MixedCaseAlignedDNAFASTAFormat,
     MixedCaseAlignedDNASequencesDirectoryFormat,
-    MixedCaseAlignedRNAFASTAFormat, MixedCaseAlignedRNASequencesDirectoryFormat
+    MixedCaseAlignedRNAFASTAFormat,
+    MixedCaseAlignedRNASequencesDirectoryFormat,
+    SequenceCharacteristicsDirectoryFormat, SequenceCharacteristicsFormat
 )
 from qiime2.plugin.testing import TestPluginBase
 from qiime2.plugin import ValidationError
@@ -808,6 +814,69 @@ class TestProteinFASTAFormats(TestPluginBase):
 
         format.validate()
 
+    def test_mixed_case_aligned_protein_fasta_format_validate_positive(self):
+        filepath = self.get_data_path(
+            'mixed-case-aligned-protein-sequences.fasta'
+            )
+        format = MixedCaseAlignedProteinFASTAFormat(filepath, mode='r')
+
+        format.validate()
+        format.validate('min')
+
+    def test_mixed_case_aligned_protein_fasta_format_unaligned(self):
+        filepath = self.get_data_path('mixed-case-protein-sequences.fasta')
+        format = MixedCaseAlignedProteinFASTAFormat(filepath, mode='r')
+
+        with self.assertRaisesRegex(
+                ValidationError, 'line 5 was length 95.* previous .* 70'):
+            format.validate()
+
+    def test_mixed_case_protein_fasta_format_validate_positive(self):
+        filepath = self.get_data_path('mixed-case-protein-sequences.fasta')
+        format = MixedCaseProteinFASTAFormat(filepath, mode='r')
+
+        format.validate()
+
+    def test_mixed_case_protein_fasta_format_invalid_characters(self):
+        filepath = self.get_data_path(
+            'mixed-case-aligned-protein-sequences.fasta'
+            )
+        format = MixedCaseProteinFASTAFormat(filepath, mode='r')
+
+        with self.assertRaisesRegex(
+                ValidationError,
+                "Invalid character '-' at position 0 on line 2"):
+            format.validate()
+
+    def test_mixed_case_aligned_protein_sequences_directory_format(self):
+        filepath = self.get_data_path(
+            'mixed-case-aligned-protein-sequences.fasta'
+            )
+        temp_dir = self.temp_dir.name
+        shutil.copy(filepath,
+                    os.path.join(
+                        temp_dir,
+                        'aligned-protein-sequences.fasta'
+                        )
+                    )
+        format = MixedCaseAlignedProteinSequencesDirectoryFormat(
+            temp_dir, mode='r'
+            )
+
+        format.validate()
+
+    def test_mixed_case_protein_sequences_directory_format(self):
+        filepath = self.get_data_path('mixed-case-protein-sequences.fasta')
+        temp_dir = self.temp_dir.name
+        shutil.copy(filepath,
+                    os.path.join(
+                        temp_dir, 'protein-sequences.fasta'
+                        )
+                    )
+        format = MixedCaseProteinSequencesDirectoryFormat(temp_dir, mode='r')
+
+        format.validate()
+
 
 class TestBLAST6Format(TestPluginBase):
     package = 'q2_types.feature_data.tests'
@@ -834,5 +903,37 @@ class TestBLAST6Format(TestPluginBase):
             BLAST6DirectoryFormat(temp_dir, mode='r').validate()
 
 
+class TestSequenceCharacteristicsFormat(TestPluginBase):
+    package = 'q2_types.feature_data.tests'
+
+    def test_sequence_characteristics_directory_format(self):
+        filepath = self.get_data_path('sequence_characteristics_length.tsv')
+        temp_dir = self.temp_dir.name
+        shutil.copy(filepath, os.path.join(temp_dir,
+                                           'sequence_characteristics.tsv'))
+        format = SequenceCharacteristicsDirectoryFormat(temp_dir, mode='r')
+        format.validate()
+
+    def test_sequence_characteristics_format(self):
+        filepath = self.get_data_path('sequence_characteristics_length.tsv')
+        format = SequenceCharacteristicsFormat(filepath, mode='r')
+        format.validate()
+
+    def test_sequence_characteristics_format_empty(self):
+        path = self.get_data_path('empty.tsv')
+        format = SequenceCharacteristicsFormat(path, mode='r')
+        with self.assertRaises(ValidationError) as context:
+            format.validate()
+        self.assertEqual(str(context.exception), 'File cannot be empty.')
+
+    def test_sequence_characteristics_format_only_index(self):
+        path = self.get_data_path('sequence_characteristics_only_index.tsv')
+        format = SequenceCharacteristicsFormat(path, mode='r')
+        with self.assertRaises(ValidationError) as context:
+            format.validate()
+        self.assertEqual(str(context.exception),
+                         'File needs to have at least two columns.')
+
+
 if __name__ == '__main__':
     unittest.main()
