@@ -6,16 +6,21 @@
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
 
+import filecmp
 import os.path
 import unittest
+import tempfile
 
 import pandas as pd
 import pandas.errors
+from pandas.testing import assert_frame_equal, assert_series_equal
 import biom
 import skbio
-import qiime2
 
-from pandas.testing import assert_frame_equal, assert_series_equal
+import qiime2
+from qiime2.plugin import ValidationError
+from qiime2.plugin.testing import TestPluginBase
+
 from q2_types.feature_table import BIOMV210Format
 from q2_types.feature_data import (
     TaxonomyFormat, HeaderlessTSVTaxonomyFormat, TSVTaxonomyFormat,
@@ -25,12 +30,16 @@ from q2_types.feature_data import (
     AlignedProteinFASTAFormat, RNAFASTAFormat, AlignedRNAFASTAFormat,
     RNAIterator, AlignedRNAIterator, BLAST6Format, MixedCaseDNAFASTAFormat,
     MixedCaseRNAFASTAFormat, MixedCaseAlignedDNAFASTAFormat,
-    MixedCaseAlignedRNAFASTAFormat
+    MixedCaseAlignedRNAFASTAFormat, BarcodeSequenceFastqIterator,
+    SequenceCharacteristicsFormat
 )
 from q2_types.feature_data._transformer import (
     _taxonomy_formats_to_dataframe, _dataframe_to_tsv_taxonomy_format,
-    ProteinIterator, AlignedProteinIterator)
-from qiime2.plugin.testing import TestPluginBase
+    ProteinIterator, AlignedProteinIterator
+)
+from q2_types.multiplexed_sequences import (
+    EMPSingleEndDirFmt, EMPSingleEndCasavaDirFmt
+)
 
 
 # NOTE: these tests are fairly high-level and mainly test the transformer
@@ -670,8 +679,10 @@ class TestDNAFASTAFormatTransformers(TestPluginBase):
 
     def test_dnafasta_format_with_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*SEQUENCE1'):
-            self.transform_format(DNAFASTAFormat, pd.Series,
-                                  'dna-sequences-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(DNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'dna-sequences-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_dnafasta_format_to_metadata(self):
         _, obs = self.transform_format(DNAFASTAFormat, qiime2.Metadata,
@@ -748,6 +759,47 @@ class TestDNAFASTAFormatTransformers(TestPluginBase):
         for observed, expected in zip(obs, exp):
             self.assertEqual(observed, expected)
 
+    def test_biom_table_to_dna_fasta_format(self):
+        table_fn = 'feature-table-with-sequence-metadata.biom'
+        input, obs = self.transform_format(
+            BIOMV210Format, DNAFASTAFormat, table_fn
+        )
+
+        self.assertIsInstance(obs, DNAFASTAFormat)
+
+        table_fp = self.get_data_path(table_fn)
+        table = biom.load_table(table_fp)
+
+        with open(str(obs)) as fh:
+            lines = fh.readlines()
+            fasta_header = lines[0].strip()
+            fasta_sequence = lines[1].strip()
+            num_lines = len(lines)
+
+        ids = table.ids(axis='observation')
+        # two lines (header & sequence) per feature
+        self.assertEqual(len(ids), num_lines / 2)
+
+        first_id = ids[0]
+        first_seq = table.metadata(axis='observation')[0]['Sequence']
+        self.assertEqual(fasta_header, f'>{first_id}')
+        self.assertEqual(fasta_sequence, first_seq)
+
+    def test_biom_table_to_dna_fasta_format_no_md(self):
+        table_fp = os.path.join('taxonomy', 'feature-table_v210.biom')
+        with self.assertRaisesRegex(TypeError, 'observation metadata'):
+            self.transform_format(BIOMV210Format, DNAFASTAFormat, table_fp)
+
+    def test_biom_table_to_dna_fasta_format_no_md_sequence_key(self):
+        table_fp = os.path.join(
+            'taxonomy', 'feature-table-with-taxonomy-metadata_v210.biom'
+        )
+
+        with self.assertRaisesRegex(
+            ValueError, 'Observation .* does not have a sequence key.*'
+        ):
+            self.transform_format(BIOMV210Format, DNAFASTAFormat, table_fp)
+
 
 class TestRNAFASTAFormatTransformers(TestPluginBase):
     package = 'q2_types.feature_data.tests'
@@ -851,8 +903,10 @@ class TestRNAFASTAFormatTransformers(TestPluginBase):
 
     def test_rnafasta_format_with_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*RNASEQUENCE1'):
-            self.transform_format(RNAFASTAFormat, pd.Series,
-                                  'rna-sequences-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(RNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'rna-sequences-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_rnafasta_format_to_metadata(self):
         _, obs = self.transform_format(RNAFASTAFormat, qiime2.Metadata,
@@ -961,9 +1015,11 @@ class TestMixedCaseDNAFASTAFormatTransformers(TestPluginBase):
 
     def test_mixed_case_dna_fasta_format_with_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*SEQUENCE1'):
-            self.transform_format(
-                        MixedCaseDNAFASTAFormat, pd.Series,
-                        'dna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(
+                        MixedCaseDNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'dna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_mixed_case_dna_fasta_format_to_metadata(self):
         _, obs = self.transform_format(MixedCaseDNAFASTAFormat,
@@ -1027,9 +1083,11 @@ class TestMixedCaseRNAFASTAFormatTransformers(TestPluginBase):
 
     def test_mixed_case_rna_fasta_format_with_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*SEQUENCE1'):
-            self.transform_format(
-                        MixedCaseRNAFASTAFormat, pd.Series,
-                        'rna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(
+                        MixedCaseRNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'rna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_mixed_case_rna_fasta_format_to_metadata(self):
         _, obs = self.transform_format(MixedCaseRNAFASTAFormat,
@@ -1095,9 +1153,11 @@ class TestMixedCaseAlignedDNAFASTAFormatTransformers(TestPluginBase):
 
     def test_mixed_case_aln_dna_fasta_format_w_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*SEQUENCE1'):
-            self.transform_format(
-                        MixedCaseAlignedDNAFASTAFormat, pd.Series,
-                        'dna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(
+                        MixedCaseAlignedDNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'dna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_mixed_case_aln_dna_fasta_format_to_metadata(self):
         _, obs = self.transform_format(
@@ -1165,9 +1225,11 @@ class TestMixedCaseAlignedRNAFASTAFormatTransformers(TestPluginBase):
 
     def test_mixed_case_aln_rna_fasta_format_w_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*SEQUENCE1'):
-            self.transform_format(
-                        MixedCaseAlignedRNAFASTAFormat, pd.Series,
-                        'rna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer = self.get_transformer(
+                MixedCaseAlignedRNAFASTAFormat, pd.Series)
+            input = self.get_data_path(
+                'rna-sequences-mixed-case-with-duplicate-ids.fasta')
+            transformer(input)
 
     def test_mixed_case_aln_rna_fasta_format_to_metadata(self):
         _, obs = self.transform_format(
@@ -1321,11 +1383,14 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
 
         obs = obs.astype(str)
 
-        index = pd.Index(['sequence1', 'sequence2'])
+        index = pd.Index(['sequence1', 'sequence2', 'sequence3'])
         exp = pd.Series(['MTTRDLTAAQFNETIQSSDMVLVDYWASWCGPCRAFAPTFAESSEK'
                          'HPDVVHAKVDTEAERELAAAAQIR',
                          'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPCKMIKPFFHSLS'
-                         'EKYSNVIFLEVDVDDCQDVASECEVKCMPTFQFFKKGQKVGEFSGAN*'],
+                         'EKYSNVIFLEVDVDDCQDVASECEVKCMPTFQFFKKGQKVGEFSGAN*',
+                         'TEPDZNZWKRUZQYTWUYKSWUQFPUNHMDBGHFDZ'
+                         'SPIYKCZHQXLCEBYJREOAUJVDLIRPEGPOGMEJ'
+                         'ZQQRHCFQXUPZLDWDGOXTOQTCIQDD*'],
                         index=index, dtype=object)
 
         assert_series_equal(exp, obs)
@@ -1333,11 +1398,14 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
     def test_series_to_proteinfasta_format(self):
         transformer = self.get_transformer(pd.Series, ProteinFASTAFormat)
 
-        index = pd.Index(['sequence1', 'sequence2'])
+        index = pd.Index(['sequence1', 'sequence2', 'sequence3'])
         input = pd.Series(['MTTRDLTAAQFNETIQSSDMVLVDYWASWCGPCRAFAPTFAESSEK'
                            'HPDVVHAKVDTEAERELAAAAQIR',
                            'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPCKMIKPFFHSLS'
-                           'EKYSNVIFLEVDVDDCQDVASECEVKCMPTFQFFKKGQKVGEFSGAN'],
+                           'EKYSNVIFLEVDVDDCQDVASECEVKCMPTFQFFKKGQKVGEFSGAN*',
+                           'TEPDZNZWKRUZQYTWUYKSWUQFPUNHMDBGHFDZ'
+                           'SPIYKCZHQXLCEBYJREOAUJVDLIRPEGPOGMEJ'
+                           'ZQQRHCFQXUPZLDWDGOXTOQTCIQDD*'],
                           index=index, dtype=object)
 
         obs = transformer(input)
@@ -1346,20 +1414,24 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
 
     def test_proteinfasta_format_with_duplicate_ids_to_series(self):
         with self.assertRaisesRegex(ValueError, 'unique.*sequence1'):
-            self.transform_format(
-                ProteinFASTAFormat,
-                pd.Series,
-                'protein-sequences-duplicate-ids.fasta')
+            transformer = self.get_transformer(ProteinFASTAFormat, pd.Series)
+            input = self.get_data_path('protein-sequences-duplicate-ids.fasta')
+            transformer(input)
 
     def test_proteinfasta_format_to_metadata(self):
         _, obs = self.transform_format(ProteinFASTAFormat, qiime2.Metadata,
                                        'protein-sequences.fasta')
-        index = pd.Index(['sequence1', 'sequence2'], name='Feature ID')
+        index = pd.Index(
+            ['sequence1', 'sequence2', 'sequence3'], name='Feature ID'
+        )
         exp_df = pd.DataFrame(['MTTRDLTAAQFNETIQSSDMVLVDYWASWCGPCRA'
                                'FAPTFAESSEKHPDVVHAKVDTEAERELAAAAQIR',
                                'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPC'
                                'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
-                               'VKCMPTFQFFKKGQKVGEFSGAN*'],
+                               'VKCMPTFQFFKKGQKVGEFSGAN*',
+                               'TEPDZNZWKRUZQYTWUYKSWUQFPUNHMDBGHFDZ'
+                               'SPIYKCZHQXLCEBYJREOAUJVDLIRPEGPOGMEJ'
+                               'ZQQRHCFQXUPZLDWDGOXTOQTCIQDD*'],
                               index=index, columns=['Sequence'], dtype=object)
         exp = qiime2.Metadata(exp_df)
 
@@ -1369,13 +1441,18 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
         _, obs = self.transform_format(AlignedProteinFASTAFormat,
                                        qiime2.Metadata,
                                        'aligned-protein-sequences.fasta')
-        index = pd.Index(['sequence1', 'sequence2'], name='Feature ID')
+        index = pd.Index(
+            ['sequence1', 'sequence2', 'sequence3'], name='Feature ID'
+        )
         exp_df = pd.DataFrame(['------------------------VDFSATWCGPC'
                                'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
                                'VKCMPTFQFFKKGQKVGEFSGAN',
                                'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPC'
                                'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
-                               'VKCMPTFQ-------VGEFSGAN'],
+                               'VKCMPTFQ-------VGEFSGAN',
+                               'MVKQIESKTAFQJALDAAGDKLVVVDFSATWCGPC'
+                               'KMIKPFFHSLSEKYSNUIFLEVDVDDCQD'
+                               'VASECEVKCMPTFO-------VGEFSGAN'],
                               index=index, columns=['Sequence'], dtype=object)
         exp = qiime2.Metadata(exp_df)
 
@@ -1387,13 +1464,16 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
 
         obs = obs.astype(str)
 
-        index = pd.Index(['sequence1', 'sequence2'])
+        index = pd.Index(['sequence1', 'sequence2', 'sequence3'])
         exp = pd.Series(['------------------------VDFSATWCGPC'
                          'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
                          'VKCMPTFQFFKKGQKVGEFSGAN',
                          'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPC'
                          'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
-                         'VKCMPTFQ-------VGEFSGAN'],
+                         'VKCMPTFQ-------VGEFSGAN',
+                         'MVKQIESKTAFQJALDAAGDKLVVVDFSATWCGPC'
+                         'KMIKPFFHSLSEKYSNUIFLEVDVDDCQD'
+                         'VASECEVKCMPTFO-------VGEFSGAN'],
                         index=index, dtype=object)
 
         assert_series_equal(exp, obs)
@@ -1402,13 +1482,16 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
         transformer = self.get_transformer(
             pd.Series, AlignedProteinFASTAFormat)
 
-        index = pd.Index(['sequence1', 'sequence2'])
+        index = pd.Index(['sequence1', 'sequence2', 'sequence3'])
         input = pd.Series(['------------------------VDFSATWCGPC'
                            'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
                            'VKCMPTFQFFKKGQKVGEFSGAN',
                            'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPC'
                            'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
-                           'VKCMPTFQ-------VGEFSGAN'],
+                           'VKCMPTFQ-------VGEFSGAN',
+                           'MVKQIESKTAFQJALDAAGDKLVVVDFSATWCGPC'
+                           'KMIKPFFHSLSEKYSNUIFLEVDVDDCQD'
+                           'VASECEVKCMPTFO-------VGEFSGAN'],
                           index=index, dtype=object)
 
         obs = transformer(input)
@@ -1426,6 +1509,11 @@ class TestProteinFASTAFormatTransformers(TestPluginBase):
                          'MVKQIESKTAFQEALDAAGDKLVVVDFSATWCGPC'
                          'KMIKPFFHSLSEKYSNVIFLEVDVDDCQDVASECE'
                          'VKCMPTFQ-------VGEFSGAN\n')
+        self.assertEqual(obs_lines[4], '>sequence3\n')
+        self.assertEqual(obs_lines[5],
+                         'MVKQIESKTAFQJALDAAGDKLVVVDFSATWCGPC'
+                         'KMIKPFFHSLSEKYSNUIFLEVDVDDCQD'
+                         'VASECEVKCMPTFO-------VGEFSGAN\n')
 
     def test_aligned_protein_fasta_format_to_protein_iterator(self):
         input, obs = self.transform_format(
@@ -1480,5 +1568,284 @@ class TestBLAST6Transformer(TestPluginBase):
         assert_frame_equal(obs.to_dataframe(), exp)
 
 
+class TestTransformers(TestPluginBase):
+    package = 'q2_types.feature_data.tests'
+
+    def setUp(self):
+        # TODO generalize plugin lookup when ported to framework. This code
+        # is adapted from the base class.
+        try:
+            from q2_types.plugin_setup import plugin
+        except ImportError:
+            self.fail("Could not import plugin object.")
+
+        self.plugin = plugin
+
+        # TODO use qiime temp dir when ported to framework, and when the
+        # configurable temp dir exists
+        self.temp_dir = tempfile.TemporaryDirectory(
+            prefix='q2-types-test-temp-')
+
+    def test_emp_multiplexed_format_barcode_sequence_iterator(self):
+        transformer = self.get_transformer(EMPSingleEndDirFmt,
+                                           BarcodeSequenceFastqIterator)
+        dirname = 'emp_multiplexed'
+        dirpath = self.get_data_path(dirname)
+        bsi = transformer(EMPSingleEndDirFmt(dirpath, mode='r'))
+        bsi = list(bsi)
+        self.assertEqual(len(bsi), 250)
+        self.assertEqual(
+            bsi[0][0],
+            ('@M00176:17:000000000-A0CNA:1:1:15487:1773 1:N:0:0',
+             'TTAGGCATCTCG',
+             '+',
+             'B@@FFFFFHHHH'))
+        self.assertEqual(
+            bsi[0][1],
+            ('@M00176:17:000000000-A0CNA:1:1:15487:1773 1:N:0:0',
+             'GCTTAGGGATTTTATTGTTATCAGGGTTAATCGTGCCAAGAAAAGCGGCATGGTCAATATAAC'
+             'CAGTAGTGTTAACAGTCGGGAGAGGAGTGGCATTAACACCATCCTTCATGAACTTAATCCACT'
+             'GTTCACCATAAACGTGACGATGAGG',
+             '+',
+             'C@CFFFFFHHFHHGIJJ?FFHEIIIIHGEIIFHGIIJHGIGBGB?DHIIJJJJCFCHIEGIGG'
+             'HGFAEDCEDBCCEEA.;>?BB=288A?AB709@:3:A:C88CCD@CC444@>>34>>ACC:?C'
+             'CD<CDCA>A@A>:<?B@?<((2(>?'))
+
+    def test_emp_se_multiplexed_format_barcode_sequence_iterator(self):
+        transformer1 = self.get_transformer(EMPSingleEndCasavaDirFmt,
+                                            EMPSingleEndDirFmt)
+        transformer2 = self.get_transformer(EMPSingleEndDirFmt,
+                                            BarcodeSequenceFastqIterator)
+        dirname = 'emp_multiplexed_single_end'
+        dirpath = self.get_data_path(dirname)
+        emp_demultiplexed = \
+            transformer1(EMPSingleEndCasavaDirFmt(dirpath, mode='r'))
+        bsi = transformer2(EMPSingleEndDirFmt(emp_demultiplexed, mode='r'))
+        bsi = list(bsi)
+        self.assertEqual(len(bsi), 250)
+        self.assertEqual(
+            bsi[0][0],
+            ('@M00176:17:000000000-A0CNA:1:1:15487:1773 1:N:0:0',
+             'TTAGGCATCTCG',
+             '+',
+             'B@@FFFFFHHHH'))
+        self.assertEqual(
+            bsi[0][1],
+            ('@M00176:17:000000000-A0CNA:1:1:15487:1773 1:N:0:0',
+             'GCTTAGGGATTTTATTGTTATCAGGGTTAATCGTGCCAAGAAAAGCGGCATGGTCAATATAAC'
+             'CAGTAGTGTTAACAGTCGGGAGAGGAGTGGCATTAACACCATCCTTCATGAACTTAATCCACT'
+             'GTTCACCATAAACGTGACGATGAGG',
+             '+',
+             'C@CFFFFFHHFHHGIJJ?FFHEIIIIHGEIIFHGIIJHGIGBGB?DHIIJJJJCFCHIEGIGG'
+             'HGFAEDCEDBCCEEA.;>?BB=288A?AB709@:3:A:C88CCD@CC444@>>34>>ACC:?C'
+             'CD<CDCA>A@A>:<?B@?<((2(>?'))
+
+    def test_invalid(self):
+        dirname = 'bad'
+        dirpath = self.get_data_path(dirname)
+        transformer = self.get_transformer(EMPSingleEndDirFmt,
+                                           BarcodeSequenceFastqIterator)
+        with self.assertRaises(ValidationError):
+            transformer(EMPSingleEndDirFmt(dirpath, mode='r'))
+
+        transformer = self.get_transformer(EMPSingleEndCasavaDirFmt,
+                                           EMPSingleEndDirFmt)
+        with self.assertRaises(ValidationError):
+            transformer(EMPSingleEndCasavaDirFmt(dirpath, 'r'))
+
+
+class BarcodeSequenceFastqIteratorTests(unittest.TestCase):
+
+    def test_valid(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1 abc/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        for i, (barcode, sequence) in enumerate(bsi):
+            self.assertEqual(barcode, barcodes[i])
+            self.assertEqual(sequence, sequences[i])
+
+    def test_too_few_barcodes(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1 abc/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_too_few_sequences(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_mismatched_id(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
+                     ('@s5/1 abc/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_mismatched_description(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1 abd/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_mismatch_description_override(self):
+        barcodes = [('@s1/2 abc/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1 abd/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences,
+                                           ignore_description_mismatch=True)
+        self.assertEqual(len(list(bsi)), 4)
+
+    def test_mismatched_handles_slashes_in_id(self):
+        # mismatch is detected as being before the last slash, even if there
+        # is more than one slash
+        barcodes = [('@s1/2/2 abc/2', 'AAAA', '+', 'YYYY')]
+        sequences = [('@s1/1/1 abc/1', 'GGG', '+', 'YYY')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_mismatched_handles_slashes_in_description(self):
+        # mismatch is detected as being before the last slash, even if there
+        # is more than one slash
+        barcodes = [('@s1/2 a/2/2', 'AAAA', '+', 'YYYY')]
+        sequences = [('@s1/1 a/1/1', 'GGG', '+', 'YYY')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+    def test_no_description(self):
+        barcodes = [('@s1/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        for i, (barcode, sequence) in enumerate(bsi):
+            self.assertEqual(barcode, barcodes[i])
+            self.assertEqual(sequence, sequences[i])
+
+    def test_only_one_description(self):
+        barcodes = [('@s1/2 abc', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2 abc', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2 abc', 'AACC', '+', 'PPPP'),
+                    ('@s4/2 abc', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1', 'GGG', '+', 'YYY'),
+                     ('@s2/1', 'CCC', '+', 'PPP'),
+                     ('@s3/1', 'AAA', '+', 'PPP'),
+                     ('@s4/1', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+        barcodes = [('@s1/2', 'AAAA', '+', 'YYYY'),
+                    ('@s2/2', 'AAAA', '+', 'PPPP'),
+                    ('@s3/2', 'AACC', '+', 'PPPP'),
+                    ('@s4/2', 'AACC', '+', 'PPPP')]
+
+        sequences = [('@s1/1 abc', 'GGG', '+', 'YYY'),
+                     ('@s2/1 abc', 'CCC', '+', 'PPP'),
+                     ('@s3/1 abc', 'AAA', '+', 'PPP'),
+                     ('@s4/1 abc', 'TTT', '+', 'PPP')]
+
+        bsi = BarcodeSequenceFastqIterator(barcodes, sequences)
+        with self.assertRaises(ValueError):
+            list(bsi)
+
+
+class TestSequenceCharacteristicsTransformer(TestPluginBase):
+    package = 'q2_types.feature_data.tests'
+
+    def setUp(self):
+        super().setUp()
+        self.exp_file = self.get_data_path(
+            'sequence_characteristics_length.tsv')
+        self.exp_df = pd.DataFrame({'length': [876, 54]},
+                                   index=pd.Index([1, 2], name='id'))
+
+    def test_df_to_sequence_characteristics_format(self):
+        transformer = self.get_transformer(pd.DataFrame,
+                                           SequenceCharacteristicsFormat)
+        obs = transformer(self.exp_df)
+
+        self.assertIsInstance(obs, SequenceCharacteristicsFormat)
+        assert filecmp.cmp(self.exp_file, obs.path)
+
+    def test_sequence_characteristics_format_to_df(self):
+        transformer = self.get_transformer(SequenceCharacteristicsFormat,
+                                           pd.DataFrame)
+        format = SequenceCharacteristicsFormat(self.exp_file, mode='r')
+        obs = transformer(format)
+
+        assert_frame_equal(self.exp_df, obs)
+
+    def test_sequence_characteristics_format_to_metadata(self):
+        transformer = self.get_transformer(SequenceCharacteristicsFormat,
+                                           qiime2.Metadata)
+        format = SequenceCharacteristicsFormat(self.exp_file, mode='r')
+        obs = transformer(format)
+
+        self.exp_df.index = pd.Index(self.exp_df.index.astype(str))
+        self.exp_df['length'] = self.exp_df['length'].astype('float64')
+
+        assert_frame_equal(obs.to_dataframe(), self.exp_df)
+
+
 if __name__ == '__main__':
     unittest.main()
