@@ -7,24 +7,26 @@
 # ----------------------------------------------------------------------------
 
 import os
-import gzip
+import re
 import itertools
 import collections
 import pathlib
+import subprocess
 
 import pandas as pd
 import skbio
 import skbio.io
 import yaml
+
 import qiime2
 import qiime2.plugin.model as model
 from qiime2.plugin import ValidationError
 
+from q2_types.bowtie2 import Bowtie2IndexDirFmt
+from q2_types.feature_data import DNAFASTAFormat
 from ..plugin_setup import plugin
-from ._util import (
-    _parse_sequence_filename,
-    _manifest_to_df,
-)
+from ._util import _parse_sequence_filename, _manifest_to_df
+from .._util import FastqGzFormat
 
 
 class FastqAbsolutePathManifestFormatV2(model.TextFileFormat):
@@ -228,62 +230,6 @@ class YamlFormat(model.TextFileFormat):
         return True
 
 
-class FastqGzFormat(model.BinaryFileFormat):
-    """
-    A gzipped fastq file.
-
-    """
-
-    def _check_n_records(self, n=None):
-        with gzip.open(str(self), mode='rt', encoding='ascii') as fh:
-            zipper = itertools.zip_longest(*[fh] * 4)
-            if n is None:
-                file_ = enumerate(zipper)
-            else:
-                file_ = zip(range(1, n), zipper)
-            for i, record in file_:
-                header, seq, sep, qual = record
-
-                if not header.startswith('@'):
-                    raise ValidationError('Header on line %d is not FASTQ, '
-                                          'records may be misaligned' %
-                                          (i * 4 + 1))
-
-                if seq is None or seq == '\n':
-                    raise ValidationError('Missing sequence for record '
-                                          'beginning on line %d'
-                                          % (i * 4 + 1))
-                elif not seq.isupper():
-                    raise ValidationError('Lowercase case sequence on line %d'
-                                          % (i * 4 + 2))
-
-                if sep is None:
-                    raise ValidationError('Missing separator for record '
-                                          'beginning on line %d'
-                                          % (i * 4 + 1))
-                elif not sep.startswith('+'):
-                    raise ValidationError('Invalid separator on line %d'
-                                          % (i * 4 + 3))
-
-                if qual is None:
-                    raise ValidationError('Missing quality for record '
-                                          'beginning on line %d'
-                                          % (i * 4 + 1))
-                elif len(qual) != len(seq):
-                    raise ValidationError('Quality score length doesn\'t '
-                                          'match sequence length for record '
-                                          'beginning on line %d'
-                                          % (i * 4 + 1))
-
-    def _validate_(self, level):
-        with self.open() as fh:
-            if fh.peek(2)[:2] != b'\x1f\x8b':
-                raise ValidationError('File is uncompressed')
-
-        record_count_map = {'min': 5, 'max': None}
-        self._check_n_records(record_count_map[level])
-
-
 class CasavaOneEightSingleLanePerSampleDirFmt(model.DirectoryFormat):
     _CHECK_PAIRED = True
     _REQUIRE_PAIRED = False
@@ -507,6 +453,225 @@ QIIME1DemuxDirFmt = model.SingleFileDirectoryFormat(
     'QIIME1DemuxDirFmt', 'seqs.fna', QIIME1DemuxFormat)
 
 
+# TODO: that's a copy of the _FastqManifestBase from q2-types
+# without the direction check. That could potentially be generalised.
+class _FastaManifestBase(model.TextFileFormat):
+    """
+    Base class for mapping of sample and mag identifiers to filepaths.
+
+    """
+    EXPECTED_HEADER = None
+    PATH_HEADER_LABEL = None
+
+    def _check_n_records(self, root, n=None):
+        with self.open() as fh:
+            header = None
+            records_seen = 0
+            file_ = enumerate(fh) if n is None else zip(range(n), fh)
+            for i, line in file_:
+                i = i + 1  # For easier reporting
+                if line.lstrip(' ') == '\n':
+                    continue  # Blank line
+                elif line.startswith('#'):
+                    continue  # Comment line
+
+                cells = [c.strip() for c in line.rstrip('\n').split(',')]
+                if header is None:
+                    if cells != self.EXPECTED_HEADER:
+                        raise ValidationError(
+                            'Found header on line %d with the following '
+                            'labels: %s, expected: %s'
+                            % (i, cells, self.EXPECTED_HEADER))
+                    else:
+                        header = cells
+                else:
+                    if len(cells) != len(header):
+                        raise ValidationError(
+                            'Line %d has %s cells (%s), expected %s.'
+                            % (i, len(cells), cells, len(header)))
+
+                    # Structure checks out, so let's make lookup easy
+                    cells = dict(zip(header, cells))
+
+                    # TODO: a bunch of tests in this subpackage aren't well
+                    # behaved --- many tests fail on this check because the
+                    # test data isn't constructed correctly. As well, there
+                    # appear to be framework-related issues preventing us from
+                    # making this kind of validation work for the relative
+                    # manifest formats at this time.
+                    if root == '':
+                        fp = os.path.join(root, cells[self.PATH_HEADER_LABEL])
+                        if not os.path.exists(os.path.expandvars(fp)):
+                            raise ValidationError(
+                                'File referenced on line %d could not be '
+                                'found (%s).'
+                                % (i, fp))
+
+                    records_seen += 1
+
+            if header is None:
+                raise ValidationError('No header found, expected: %s.'
+                                      % self.EXPECTED_HEADER)
+
+            if records_seen == 0:
+                raise ValidationError('No sample records found in manifest, '
+                                      'only observed comments, blank lines, '
+                                      'and/or a header row.')
+
+
+class MultiMAGManifestFormat(_FastaManifestBase):
+    EXPECTED_HEADER = ['sample-id', 'mag-id', 'filename']
+    PATH_HEADER_LABEL = 'filename'
+
+    def _validate_(self, level):
+        self._check_n_records(root=str(self.path.parent),
+                              n={'min': 10, 'max': None}[level])
+
+
+class MultiDirValidationMixin:
+    def _validate_(self, level):
+        for p in self.path.iterdir():
+            if not p.is_dir() and p.name not in ['MANIFEST']:
+                raise ValidationError(
+                    "Files should be organised in per-sample directories")
+
+
+class MultiFASTADirectoryFormat(MultiDirValidationMixin,
+                                model.DirectoryFormat):
+    pathspec = r'.+\.(fa|fasta)$'
+    sequences = model.FileCollection(pathspec, format=DNAFASTAFormat)
+
+    @sequences.set_path_maker
+    def sequences_path_maker(self, sample_id, mag_id):
+        # write out with fasta extension, regardless if input was fa or fasta
+        return '%s/%s.fasta' % (sample_id, mag_id)
+
+    def sample_dict(self, relative=False):
+        """
+        Returns a mapping of sample id to another dictionary where keys
+        represent the MAG ID and values correspond to the filepath for
+        each MAG.
+
+        Parameters
+        ---------
+        relative : bool
+            Whether to return filepaths relative to the directory's location.
+            Returns absolute filepaths by default.
+
+        Returns
+        -------
+        dict
+            Mapping of sample id -> dict {mag_id: mag_filepath} as
+            described above. Both levels of the dictionary are
+            sorted alphabetically by key.
+        """
+        mags_pattern = re.compile(self.pathspec)
+        ids = {}
+        for d in self.path.iterdir():
+            if not d.is_dir():
+                continue
+
+            sample_id = d.name.rsplit('/', 1)[0]
+            if sample_id not in ids:
+                ids[sample_id] = {}
+
+            for path in d.iterdir():
+                if not mags_pattern.match(path.name):
+                    continue
+
+                mag_id = os.path.splitext(os.path.basename(path.name))[0]
+                absolute_path = path.absolute()
+                if relative:
+                    ids[sample_id][mag_id] = str(
+                        absolute_path.relative_to(self.path.absolute())
+                    )
+                else:
+                    ids[sample_id][mag_id] = str(absolute_path)
+
+            ids[sample_id] = dict(sorted(ids[sample_id].items()))
+
+        return dict(sorted(ids.items()))
+
+
+class MultiMAGSequencesDirFmt(MultiFASTADirectoryFormat):
+    manifest = model.File('MANIFEST', format=MultiMAGManifestFormat)
+
+
+class MultiBowtie2IndexDirFmt(MultiDirValidationMixin, Bowtie2IndexDirFmt):
+    pass
+
+
+class ContigSequencesDirFmt(model.DirectoryFormat):
+    pathspec = r'[^\.].+_contigs.(fasta|fa)$'
+
+    sequences = model.FileCollection(pathspec, format=DNAFASTAFormat)
+
+    @sequences.set_path_maker
+    def sequences_path_maker(self, sample_id):
+        return r'%s_contigs.fasta' % sample_id
+
+    def sample_dict(self, relative=False):
+        '''
+        Returns a mapping of sample id to filepath for each set of per-sample
+        contigs.
+
+        Parameters
+        ---------
+        relative : bool
+            Whether to return filepaths relative to the directory's location.
+            Returns absolute filepaths by default.
+
+        Returns
+        -------
+        dict
+            Mapping of sample id -> filepath as described above. Sorted
+            alphabetically by key.
+        '''
+        contigs_pattern = re.compile(self.pathspec)
+        ids = {}
+        for path in self.path.iterdir():
+            if not contigs_pattern.match(path.name):
+                continue
+
+            id = path.name.rsplit('_contigs', 1)[0]
+            absolute_path = path.absolute()
+            if relative:
+                ids[id] = str(
+                    absolute_path.relative_to(self.path.absolute())
+                )
+            else:
+                ids[id] = str(absolute_path)
+
+        return dict(sorted(ids.items()))
+
+
+# borrowed from q2-phylogenomics
+class BAMFormat(model.BinaryFileFormat):
+    def _validate_(self, level):
+        cmd = ['samtools', 'quickcheck', '-v', str(self)]
+        result = subprocess.run(cmd)
+        if result.returncode != 0:
+            raise model.ValidationError(
+                'samtools quickcheck -v failed on %s' % self.path.name)
+
+
+# borrowed from q2-phylogenomics
+class BAMDirFmt(model.DirectoryFormat):
+    bams = model.FileCollection(r'.+\.bam', format=BAMFormat)
+
+    @bams.set_path_maker
+    def bams_path_maker(self, sample_id):
+        return '%s.bam' % sample_id
+
+
+class MultiBAMDirFmt(MultiDirValidationMixin, model.DirectoryFormat):
+    bams = model.FileCollection(r'.+\/.+\.bam', format=BAMFormat)
+
+    @bams.set_path_maker
+    def bams_path_maker(self, sample_id, genome_id):
+        return '%s/%s.bam' % sample_id, genome_id
+
+
 plugin.register_formats(
     FastqManifestFormat, YamlFormat, FastqGzFormat,
     CasavaOneEightSingleLanePerSampleDirFmt,
@@ -517,5 +682,7 @@ plugin.register_formats(
     PairedEndFastqManifestPhred64, SingleEndFastqManifestPhred33V2,
     SingleEndFastqManifestPhred64V2, PairedEndFastqManifestPhred33V2,
     PairedEndFastqManifestPhred64V2, QIIME1DemuxFormat, QIIME1DemuxDirFmt,
-    SampleIdIndexedSingleEndPerSampleDirFmt
+    SampleIdIndexedSingleEndPerSampleDirFmt, MultiFASTADirectoryFormat,
+    MultiMAGSequencesDirFmt, ContigSequencesDirFmt, MultiBowtie2IndexDirFmt,
+    BAMDirFmt, MultiBAMDirFmt
 )
