@@ -6,16 +6,21 @@
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
 
+
 import functools
 import unittest
 import os
-import shutil
 import io
+import shutil
 import string
 
 import skbio
 import yaml
 import pandas as pd
+from pandas._testing import assert_frame_equal
+
+from qiime2.plugin import ValidationError
+from qiime2.plugin.testing import TestPluginBase
 
 from q2_types.per_sample_sequences import (
     SingleLanePerSampleSingleEndFastqDirFmt,
@@ -24,25 +29,27 @@ from q2_types.per_sample_sequences import (
     CasavaOneEightLanelessPerSampleDirFmt,
     SingleEndFastqManifestPhred33,
     SingleEndFastqManifestPhred64,
-    PairedEndFastqManifestPhred33,
-    PairedEndFastqManifestPhred64,
-    FastqAbsolutePathManifestFormat,
-    FastqManifestFormat,
     SingleEndFastqManifestPhred33V2,
     SingleEndFastqManifestPhred64V2,
+    PairedEndFastqManifestPhred33,
+    PairedEndFastqManifestPhred64,
     PairedEndFastqManifestPhred33V2,
     PairedEndFastqManifestPhred64V2,
+    FastqAbsolutePathManifestFormat,
+    FastqManifestFormat,
     QIIME1DemuxDirFmt,
     FastqGzFormat,
-    SampleIdIndexedSingleEndPerSampleDirFmt)
+    SampleIdIndexedSingleEndPerSampleDirFmt,
+    MultiFASTADirectoryFormat,
+    MultiMAGManifestFormat,
+    MultiMAGSequencesDirFmt
+)
 from q2_types.per_sample_sequences._util import (
     _validate_header,
     _validate_single_end_fastq_manifest_directions,
     _validate_paired_end_fastq_manifest_directions,
     _parse_and_validate_manifest
 )
-from qiime2.plugin import ValidationError
-from qiime2.plugin.testing import TestPluginBase
 
 
 _parse_and_validate_manifest_partial = functools.partial(
@@ -137,7 +144,7 @@ class TestTransformers(TestPluginBase):
 
         input, dirfmt = self.transform_format(
             CasavaOneEightLanelessPerSampleDirFmt,
-            SingleLanePerSamplePairedEndFastqDirFmt, filenames=filenames
+            SingleLanePerSampleSingleEndFastqDirFmt, filenames=filenames
         )
         expected_filepaths = ['Human-Kneecap_S1_L001_R1_001.fastq.gz',
                               'Human-Armpit_S2_L001_R1_001.fastq.gz']
@@ -172,7 +179,7 @@ class TestTransformers(TestPluginBase):
         filenames = ('Human-Kneecap_S1_L001_R1_001.fastq.gz',)
         input, obs = self.transform_format(
             CasavaOneEightSingleLanePerSampleDirFmt,
-            SingleLanePerSamplePairedEndFastqDirFmt, filenames=filenames
+            SingleLanePerSampleSingleEndFastqDirFmt, filenames=filenames
         )
 
         input = skbio.io.read(
@@ -209,7 +216,7 @@ class TestTransformers(TestPluginBase):
     def test_miseq_demux_dirfmt_to_slpspefdf(self):
         input, obs = self.transform_format(
             CasavaOneEightLanelessPerSampleDirFmt,
-            SingleLanePerSamplePairedEndFastqDirFmt,
+            SingleLanePerSampleSingleEndFastqDirFmt,
             filenames=('Human-Kneecap_S1_R1_001.fastq.gz',),
         )
 
@@ -1188,5 +1195,83 @@ class TestFastqManifestV2Transformers(TestPluginBase):
             self.assertEqual(obs_manifest.read(), self.exp_pe_manifest)
 
 
+class TestMAGTransformers(TestPluginBase):
+    package = "q2_types.per_sample_sequences.tests"
+
+    def setUp(self):
+        super().setUp()
+
+    @staticmethod
+    def construct_manifest(ext):
+        exp_manifest = (
+            "sample-id,mag-id,filename\n"
+            f"sample1,mag1,sample1/mag1.{ext}\n"
+            f"sample1,mag2,sample1/mag2.{ext}\n"
+            f"sample1,mag3,sample1/mag3.{ext}\n"
+            f"sample2,mag1,sample2/mag1.{ext}\n"
+            f"sample2,mag2,sample2/mag2.{ext}\n"
+        )
+        return exp_manifest
+
+    def apply_transformation(self, from_fmt, to_fmt, datafile_fp):
+        transformer = self.get_transformer(from_fmt, to_fmt)
+        fp = self.get_data_path(datafile_fp)
+        return transformer(from_fmt(fp, 'r'))
+
+    def test_multifile_dirfmt_to_mag_seqs_dirfmt_fa(self):
+        obs = self.apply_transformation(
+            MultiFASTADirectoryFormat,
+            MultiMAGSequencesDirFmt,
+            'mags/mags-fa'
+        )
+        with obs.manifest.view(MultiMAGManifestFormat).open() as obs_manifest:
+            self.assertEqual(
+                obs_manifest.read(), self.construct_manifest('fasta')
+            )
+
+    def test_multifile_dirfmt_to_mag_seqs_dirfmt_fasta(self):
+        obs = self.apply_transformation(
+            MultiFASTADirectoryFormat,
+            MultiMAGSequencesDirFmt,
+            'mags/mags-fasta'
+        )
+        with obs.manifest.view(MultiMAGManifestFormat).open() as obs_manifest:
+            self.assertEqual(
+                obs_manifest.read(), self.construct_manifest('fasta')
+            )
+
+    def test_mag_manifest_to_df(self):
+        obs = self.apply_transformation(
+            MultiMAGManifestFormat,
+            pd.DataFrame,
+            'manifests/MANIFEST-mags-fa'
+        )
+        exp = pd.DataFrame({
+            'sample-id': [
+                'sample1', 'sample1', 'sample1', 'sample2', 'sample2'
+            ],
+            'mag-id': ['mag1', 'mag2', 'mag3', 'mag1', 'mag2'],
+            'filename': [
+                os.path.join(self.get_data_path('manifests'), x)
+                for x in [
+                    'sample1/mag1.fasta', 'sample1/mag2.fasta',
+                    'sample1/mag3.fasta', 'sample2/mag1.fasta',
+                    'sample2/mag2.fasta'
+                ]
+            ]
+        })
+        exp.set_index(['sample-id', 'mag-id'], inplace=True)
+
+        assert_frame_equal(exp, obs)
+
+    def test_mag_seqs_dirfmt_to_multifile_dirfmt(self):
+        obs = self.apply_transformation(
+            MultiMAGSequencesDirFmt,
+            MultiFASTADirectoryFormat,
+            'mags/mags-fa-with-manifest'
+        )
+        obs.validate()
+
+
 if __name__ == '__main__':
     unittest.main()
